@@ -1,8 +1,16 @@
 /*  Rational - Rational number class with overflow detection
-    Copyright (C) 2012-2016 Antonio Diaz Diaz.
+    Copyright (C) 2012-2017 Antonio Diaz Diaz.
 
-    This program is free software: you have unlimited permission to
-    copy, distribute and modify it.
+    This program is free software. Redistribution and use in source and
+    binary forms, with or without modification, are permitted provided
+    that the following conditions are met:
+
+    1. Redistributions of source code must retain the above copyright
+    notice, this list of conditions and the following disclaimer.
+
+    2. Redistributions in binary form must reproduce the above copyright
+    notice, this list of conditions and the following disclaimer in the
+    documentation and/or other materials provided with the distribution.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,7 +39,7 @@ namespace {
 
 const char * const Program_name = "Rational";
 const char * const program_name = "rational";
-const char * const program_year = "2016";
+const char * const program_year = "2017";
 const char * invocation_name = 0;
 
 
@@ -58,7 +66,8 @@ void show_version()
   {
   std::printf( "%s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "This is free software: you are free to change and redistribute it.\n"
+  std::printf( "License 2-clause BSD.\n"
+               "This is free software: you are free to change and redistribute it.\n"
                "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
@@ -87,22 +96,23 @@ void internal_error( const char * const msg )
 
 void approximate_pi()
   {
-  const std::string target( "3.14159265358979323846264338327950288419" );
+  const std::string target( "3.141592653589793238462643383279502884197" );
   const int tlen = target.size();
   int max_len = 1;
-  int n = 3, d = 1, digit = 0;
+  int n = 3, d = 1, dif = 0;
 
   std::printf( "target == %s\n", target.c_str() );
   std::fputs( "correct\ndecimals                  value\n", stdout );
-  while( n < INT_MAX )
+  while( n <= INT_MAX - 3 )
     {
-    if( n / d != 3 ) { if( n / d < 3 ) ++n; else ++d; continue; }
+    const int q = n / d;
+    if( q != 3 ) { if( q < 3 ) ++n; else { n += 3; ++d; } continue; }
     long long rest = n % d;
     int i = 2;
     for( ; i < tlen; ++i )
       {
-      rest *= 10; digit = '0' + ( rest / d ); rest %= d;
-      if( digit != target[i] ) break;
+      rest *= 10; dif = '0' + ( rest / d ) - target[i]; rest %= d;
+      if( dif != 0 ) break;
       }
     if( i > max_len )
       {
@@ -114,8 +124,7 @@ void approximate_pi()
       std::fflush( stdout );
       if( max_len >= tlen ) break;
       }
-    if( digit < target[i] ) ++n;
-    else ++d;
+    if( dif < 0 ) ++n; else { n += 3; ++d; }
     }
   }
 
@@ -225,16 +234,16 @@ int run_tests( const int level, const bool verbose )
     { std::fputs( "error: Rational( INT_MAX, INT_MAX ) not accepted.\n", stderr );
       retval = 1; }
   Rational r( INT_MIN, INT_MIN );
-  if( !r.error() )
-    { std::fputs( "error: Rational( INT_MIN, INT_MIN ) not rejected.\n", stderr );
+  if( r != 1 || r != r0 )
+    { std::fputs( "error: Rational( INT_MIN, INT_MIN ) not accepted.\n", stderr );
       retval = 1; }
   r.assign( INT_MAX, INT_MAX );
   if( r != 1 || r != r0 )
     { std::fputs( "error: r.assign( INT_MAX, INT_MAX ) not accepted.\n", stderr );
       retval = 1; }
   r.assign( INT_MIN, INT_MIN );
-  if( !r.error() )
-    { std::fputs( "error: r.assign( INT_MIN, INT_MIN ) not rejected.\n", stderr );
+  if( r != 1 || r != r0 )
+    { std::fputs( "error: r.assign( INT_MIN, INT_MIN ) not accepted.\n", stderr );
       retval = 1; }
   r.assign( INT_MAX, -INT_MAX );
   if( r != -1 || r != -r0 || -r != r0 )
@@ -279,6 +288,24 @@ int run_tests( const int level, const bool verbose )
   const Rational r2( 0 );
   if( r1 != r2 || r1 != -r2 || r1 + r2 != 0 || r1 - r2 != 0 || r1 * r2 != 0 )
     { std::fputs( "error: ( 0, 1 ) != ( 0 )\n", stderr ); retval = 1; }
+  const Rational r3( r2.inverse() );
+  const Rational r4( 1, 0 );
+  if( !r3.error() || !r4.error() || r3.numerator() != r4.numerator() ||
+      r3.denominator() != r4.denominator() )
+    { std::fprintf( stderr, "error: inverse( 0 ) [%d,%d] != Rational( 1, 0 ) [%d,%d]\n",
+      r3.numerator(), r3.denominator(), r4.numerator(), r4.denominator() );
+      retval = 1; }
+  Rational r;
+  if( r.numerator() != r2.numerator() || r.denominator() != r2.denominator() )
+    { std::fprintf( stderr, "error: Rational() [%d,%d] != Rational( 0 ) [%d,%d]\n",
+      r.numerator(), r2.denominator(), r.numerator(), r2.denominator() );
+      retval = 1; }
+  r.assign( 1, 0 );
+  if( !r.error() || r.numerator() != r4.numerator() ||
+      r.denominator() != r4.denominator() )
+    { std::fprintf( stderr, "error: assign( 1, 0 ) [%d,%d] != Rational( 1, 0 ) [%d,%d]\n",
+      r.numerator(), r.denominator(), r4.numerator(), r4.denominator() );
+      retval = 1; }
   }
 
   // intermediate overflow test
@@ -327,9 +354,10 @@ int run_tests( const int level, const bool verbose )
   // multiplication by inverse test
   for( int n = -mul_limit; n <= mul_limit; ++n )
     {
+    if( n == 0 ) continue;
     const Rational r1( n );
     const Rational r2( r1.inverse() );
-    if( n != 0 && r1 * r2 != 1 )
+    if( r1 * r2 != 1 )
       { std::fprintf( stderr, "error: %d * 1/%d != 1\n", n, n );
         retval = 1; break; }
     }
